%%%%% solving huggett's model by value function iteration
tic
clearvars;
close all;
clc;

%%%%% economic parameters

beta  = 0.95;      %% time discount factor   
alpha = 1.5;       %% CRRA (=1/IES)

%%%%% 2-state markov chain for endowments

ymin  = 0.1; 
ymax  = 1.0;
ygrid = [ymin;ymax];

p11 = 0.500; p12 = 1 - p11;
p22 = 0.925; p21 = 1 - p22;

P   = [p11,p12;p21,p22];

%%%%% asset grid

phi  =  (ymin/(1-beta))-eps;      %% borrowing constraint

na       = 1000; 
amin     = -phi;
amax     = 12; 

agrid    = nodeunif(na , amin, amax);

%%%%% numerical parameters

max_iter = 1000;
tol      = 1e-7;
penalty  = 10^16;

%%%% state grid

s  = gridmake(agrid,ygrid); % ns-by-2 matrix where ns=na*ny
ns = size(s,1);        

a  = s(:,1);
y  = s(:,2);

%%%%% parameters structure to pass to functions

parameters.beta  = beta;
parameters.alpha = alpha;

parameters.agrid = agrid;
parameters.a     = a;
parameters.y     = y;
parameters.P     = P;

%%%%% find q that zeros out market-clearing condition

qmin  =  beta+eps;
qmax  =  1   -eps;

%fmin = findq(qmin,parameters,max_iter,penalty,tol);
%fmax = findq(qmax,parameters,max_iter,penalty,tol);

optset('bisect','tol',tol) ; 
tic
q = bisect('findq',qmin,qmax,parameters,max_iter,penalty,tol);
toc

%%%%% shape of z=F(q)
%nq    = 20;
%qgrid = linspace(qmin,qmax,nq);
%F     = zeros(nq,1);
%for i=1:nq,
%F(i) = findq(qgrid(i),parameters,max_iter,penalty,tol);
%end

%[~,arg] = min(abs(F));
%q = qgrid(arg);

%figure(4)
%set(gcf,'DefaultLineLineWidth', 1.5);
%plot(qgrid,F,qgrid,zeros(nq,1),'k--')
%set(get(gcf,'CurrentAxes'),'FontSize',14,'LineWidth', 1.5)
%set(gca,'DefaultTextInterpreter', 'latex','TickLabelInterpreter','latex') 
%xlabel('bond price, $q$','Interpreter','latex');
%ylabel('excess demand for bonds, $F(q)$','Interpreter','latex');

%%%%% given q recover other variables of interest

%%%%% reward function

c    = zeros(ns,na);

for j=1:na,
    
    c(:,j) = a + y - q*agrid(j);
  
end

%%%%% check consumption non-negative

violations = (c<0);

c = c.*(c>=0) + eps;

if alpha==1,
    
    u = log(c) - penalty*violations;
    
else
    
    u = (1/(1-alpha))*(c.^(1-alpha) - 1) - penalty*violations;

end

%%%%% initialize inner dynamic programming loop

v       = log(0.5*y)/(1-beta);
iter    = 0;
    
for i=1:max_iter,

RHS      = u+beta*kron(P,ones(na,1))*reshape(v,na,2)';

[Tv,argmax] = max(RHS,[],2);

%%%%% policy that attains the maximum

g = a(argmax);

%%%%% check if converged

error = max(abs(Tv-v));

%fprintf('%4i %6.2e \n',[i, error]);

if norm(Tv-v,inf)<tol, break, end;

%%%%% if not converged, update guess

v = Tv;

end

%%%%% construct transition matrix for the state s=(a,y)

A = zeros(ns,na);
Q = zeros(ns,ns);

PP = kron(P,ones(na,1));

for s=1:ns,
    
    A(s,:) = (agrid==g(s))';  %% puts a 1 if g(s)=a 
    
    Q(s,:) = kron(PP(s,:),A(s,:));
    
end

%%%%% compute stationary distribution

[eig_vectors,eig_values] = eig(Q'); 
[~,arg] = min(abs(diag(eig_values)-1)); 
unit_eig_vector = eig_vectors(:,arg); 

mu = unit_eig_vector/sum(unit_eig_vector); 

%%%%% check market clearing

z = sum(mu.*g);

na       = numel(agrid);
ny       = numel(ygrid);

c        = a+y-q*g;

g        = reshape(g ,na,ny);
c        = reshape(c ,na,ny);
m        = reshape(mu,na,ny);

% smooth solution using splines

fspace   = fundefn('spli',21,amin,amax,2);

N        = 5000;

curv     = 0.25;     %% (curv = 0 log-spaced, curv = 1 linear)

afine    = nodeunif(N, (amin+phi).^curv, (amax+phi).^curv).^(1/curv); % trick to avoid negative values
afine    = afine-phi;

gfine    = zeros(N,2);
cfine    = zeros(N,2);

for i=1:2,

coeff_g    = funfitxy(fspace, agrid, g(:,i));
coeff_c    = funfitxy(fspace, agrid, c(:,i));

gfine(:,i) = funeval(coeff_g, fspace, afine);
cfine(:,i) = funeval(coeff_c, fspace, afine);

end


figure(1)
set(gcf,'DefaultLineLineWidth', 1.5);
plot(afine,gfine(:,2),afine,gfine(:,1),...
     agrid,agrid,'k--',...
     agrid,zeros(size(agrid)),'k--',zeros(size(agrid)),agrid,'k--')
set(get(gcf,'CurrentAxes'),'FontSize',14,'LineWidth', 1.5)
set(gca,'DefaultTextInterpreter', 'latex','TickLabelInterpreter','latex') 
 
xlabel('asset level, $a$','Interpreter','latex');
ylabel('asset policy, $a^{\prime}=g(a,y)$','Interpreter','latex');
ha = legend('$y=y_H$', '$y=y_L$','Location','SouthEast');
set(ha,'Interpreter','latex');
axis([amin amax amin amax]) 

cmin = min(min(cfine));
cmax = max(max(cfine));

figure(2)
set(gcf,'DefaultLineLineWidth', 1.5);
plot(afine,cfine(:,2),afine,cfine(:,1),...
     agrid,zeros(size(agrid)),'k--',zeros(size(agrid)),agrid,'k--')

set(get(gcf,'CurrentAxes'),'FontSize',14,'LineWidth', 1.5)
set(gca,'DefaultTextInterpreter', 'latex','TickLabelInterpreter','latex') 
 
xlabel('asset level, $a$','Interpreter','latex');
ylabel('consumption policy, $c(a,y)$','Interpreter','latex');
ha = legend('$y=y_H$', '$y=y_L$','Location','SouthEast');
set(ha,'Interpreter','latex');
axis([amin amax cmin cmax]) 

figure(3)
set(gcf,'DefaultLineLineWidth', 1.5);
plot(agrid,m(:,1)+m(:,2))
set(get(gcf,'CurrentAxes'),'FontSize',14,'LineWidth', 1.5)
set(gca,'DefaultTextInterpreter', 'latex','TickLabelInterpreter','latex')  
xlabel('asset level, $a$','Interpreter','latex');
ylabel('marginal asset distribution, $\sum_y \mu(a,y)$','Interpreter','latex');



